
# q-expansion of newform 3200.2.c.q, downloaded from the LMFDB on 23 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3200
weight = 2
poly_data = [1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = [[1151, [1, 0]], [901, [1, 0]], [2177, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 0], [0, 1], [-6, 0], [0, 1], [0, 3], [-2, 0], [0, -3], [-6, 0], [-4, 0], [0, -3], [-2, 0], [0, -2], [0, -5], [0, 1], [10, 0], [-10, 0], [0, -2], [-16, 0], [0, -3], [0, 0], [0, 4], [-6, 0], [0, -1], [-10, 0], [0, 9], [0, -8], [10, 0], [0, -3], [0, -9], [-10, 0], [0, 11], [-6, 0], [-14, 0], [8, 0], [0, 1], [0, 4], [0, 5], [0, -9], [-10, 0], [-10, 0], [-12, 0], [0, 5], [0, -5], [-20, 0], [-10, 0], [0, -5], [0, 14], [10, 0], [0, -7], [16, 0], [-14, 0], [-6, 0], [0, 3], [0, 1], [-30, 0], [-20, 0], [0, -11], [-6, 0], [0, 10], [0, 11], [0, 10], [-16, 0], [0, -7], [0, 3], [10, 0], [0, 11], [0, 4], [18, 0], [0, -3], [-4, 0], [0, 11], [0, -13], [26, 0], [0, -9], [2, 0], [0, -5], [-2, 0], [-22, 0], [-26, 0], [30, 0], [12, 0], [0, -7], [4, 0], [0, -18], [30, 0], [0, 3], [38, 0], [0, 11], [0, 6], [-8, 0], [0, 1], [-6, 0], [-2, 0], [0, 13], [10, 0], [-6, 0], [0, 14], [-34, 0], [0, 14], [0, 1], [0, 12], [-6, 0], [26, 0], [0, -17], [0, 4], [0, 17], [12, 0], [-38, 0], [0, -9], [0, 17], [0, -17], [34, 0], [-8, 0], [-30, 0], [0, 12], [0, 9], [0, -7], [-10, 0], [22, 0], [0, -3], [0, 3], [0, -18], [30, 0], [-18, 0], [10, 0], [0, 0], [0, 1], [0, 15], [30, 0], [0, -3], [36, 0], [0, -11], [-6, 0], [-38, 0], [0, -7], [0, -2], [0, 27], [22, 0], [26, 0], [22, 0], [0, -7], [0, 4], [-6, 0], [36, 0], [0, 17], [0, 15], [-6, 0], [0, 3], [0, 17], [30, 0], [0, -20], [0, -7], [0, 12], [4, 0], [28, 0], [-18, 0], [0, -13], [14, 0], [0, -6], [0, -27], [0, -3], [34, 0], [0, -5], [0, -15], [-52, 0], [0, 3], [10, 0], [0, -23], [-30, 0], [-2, 0], [-40, 0], [0, 17], [-8, 0], [-6, 0], [-14, 0], [-18, 0], [0, 13], [10, 0], [0, -1], [30, 0], [0, 25], [0, -13], [0, -13], [-62, 0], [0, -7], [0, -28], [-22, 0], [-36, 0], [0, -23], [0, 10], [22, 0], [-66, 0], [0, -6], [0, 13], [30, 0], [0, 33], [0, -25], [0, 33], [-14, 0], [12, 0], [0, 3], [-18, 0], [-54, 0], [0, 9], [32, 0], [0, 12], [-54, 0], [-38, 0], [0, -1], [-2, 0], [0, -7], [0, -24], [-44, 0], [30, 0], [0, 23], [30, 0], [0, -19], [0, 33], [-26, 0], [44, 0], [26, 0], [0, 19], [0, -34], [2, 0], [0, 9], [-64, 0], [0, 1], [18, 0], [0, -23], [38, 0], [20, 0], [26, 0], [0, -22], [0, -9], [-18, 0], [0, -23], [10, 0], [-56, 0], [0, 12], [2, 0], [0, -7], [-38, 0], [0, -11], [-28, 0], [0, -13], [-42, 0], [-46, 0], [0, -29], [0, 11], [18, 0], [0, -27], [-26, 0], [0, -9], [-42, 0], [-2, 0], [0, 4], [0, -35], [0, 3], [0, 7], [0, 30], [50, 0], [0, -1], [0, 31], [30, 0], [-46, 0], [-18, 0], [0, -18], [0, -13], [22, 0], [0, 6], [0, -31], [-64, 0], [0, 7], [0, 1], [0, 32], [-46, 0], [10, 0], [6, 0], [0, -13], [-24, 0], [0, 33], [-26, 0], [0, -4], [-44, 0], [0, -11], [0, -13], [60, 0], [-38, 0], [22, 0], [0, -18], [0, 9], [-30, 0], [0, 7], [-14, 0], [-20, 0], [0, 19], [-6, 0], [0, -6], [0, -27], [0, 9], [-64, 0]]
